@@ -76,13 +76,13 @@ ht
 ## ---- echo = FALSE-------------------------------------------------------
 sides <- c('left_', 'right_', 'top_', 'bottom_')
 props <- list()
-props[['Cell_Text']] <- sort(c('font', 'text_color', 'wrap', 'bold', 'italic', 'font', 'font_size', 'na_string', 'pad_decimal', 'escape_contents', 'numeric_format', 'rotation'))
+props[['Cell_Text']] <- sort(c('font', 'text_color', 'wrap', 'bold', 'italic', 'font', 'font_size', 'na_string', 'pad_decimal', 'escape_contents', 'number_format', 'rotation'))
 
 props[['Cell']] <- sort(c('align', 'valign', 'rowspan', 'colspan', 'background_color', paste0(sides, 'border'),
       paste0(sides, 'border_color'), paste0(sides, 'padding')))
 props[['Row']]    <- 'row_height'
 props[['Column']] <- 'col_width'
-props[['Table']]  <- sort(c('width', 'height', 'position', 'caption', 'caption_pos', 'tabular_environment', 'label'))
+props[['Table']]  <- sort(c('width', 'height', 'position', 'caption', 'caption_pos', 'tabular_environment', 'label', 'latex_float'))
 
 maxl <- max(sapply(props, length))
 props <- lapply(props, function(x) c(x, rep('', maxl - length(x))))
@@ -165,6 +165,11 @@ car_ht <- car_ht                               %>%
 
 car_ht
 
+## ------------------------------------------------------------------------
+ht <- insert_row(ht, 'Hadley Wickham', '100000', after = 3)
+ht <- add_footnote(ht, 'DHJ deserves a pay rise')
+ht
+
 ## ------------------------------------------------------------------------
 number_format(car_ht) <- 0
 car_ht[1:5,]
@@ -179,13 +184,22 @@ pad_decimal(pointy_ht)[2:5] <- '.'
 align(pointy_ht) <- 'right'
 pointy_ht
 
+## ------------------------------------------------------------------------
+code_ht <- if (is_latex) hux('Here is some maths: $a^b$') else 
+      hux('The code for a copyright symbol is &copy;')
+code_ht
+
+## ------------------------------------------------------------------------
+escape_contents(code_ht)[1, 1] <- FALSE
+code_ht
+
 ## ------------------------------------------------------------------------
 width(ht) <- 0.35
 col_width(ht) <- c(.7, .3)
 ht
 
 ## ------------------------------------------------------------------------
-ht[4, 1] <- 'David Arthur Shrimpton Hugh-Jones'
+ht[5, 1] <- 'David Arthur Shrimpton Hugh-Jones'
 ht 
 
 ## ------------------------------------------------------------------------
@@ -267,8 +281,8 @@ where(m == 'dog') # m is equal to 'dog' in cells (1, 1), (3, 1), (4, 1) and (4,
 color_demo <- matrix('text', 7, 7)
 rainbow <- c('red', 'orange', 'yellow', 'green', 'blue', 'turquoise', 'violet')
 color_demo <- as_hux(color_demo)                  %>% 
-      set_text_color(rainbow)                     %>% # text in columns
-      set_background_color(rainbow, byrow = TRUE) %>% # background color in rows
+      set_text_color(rainbow)                     %>% # text rainbow goes down columns
+      set_background_color(rainbow, byrow = TRUE) %>% # background color rainbow goes along rows
       set_all_borders(1)                          %>% 
       set_all_border_colors('white')
 color_demo
