@@ -77,7 +77,7 @@ test_that('Column names are not uglified', {
   skip('Can\'t solve this one at the moment.')
   ht <- hux('A long column name' = 1:3, 'Another name' = 1:3, add_colnames = TRUE)
   expect_match(to_screen(ht), 'A long column name', fixed = TRUE, all = FALSE)
-  ht <- hux('A long column name' = 1:3, 'Another name' = 1:3, add_colnames = TRUE)
+  ht <- hux('A long column name' = 1:3, 'Another name' = 1:3, add_colnames = FALSE)
   ht <- huxtable::add_colnames(ht)
   expect_match(to_screen(ht), 'A long column name', fixed = TRUE, all = FALSE)
 })
@@ -122,15 +122,15 @@ test_that('insert_column and insert_row work', {
 
   ht <- insert_row(ht_orig, 8, 9, after = 1)
   expect_equivalent(nrow(ht), 3)
-  expect_equivalent(ht[, 2], huxtable(b=c(1,9,2)))
+  expect_equivalent(ht[, 2], huxtable(b = c(1, 9, 2)))
 
   ht <- insert_column(ht_orig, 8, 9)
   expect_equivalent(ncol(ht), 3)
   expect_equivalent(ht[2, 1], 9)
 
-  ht<- insert_column(ht_orig, 8, 9, after = 1)
+  ht <- insert_column(ht_orig, 8, 9, after = 1)
   expect_equivalent(ncol(ht), 3)
-  expect_equivalent(ht[1, ], huxtable(a=1, 8, b=1))
+  expect_equivalent(ht[1, ], huxtable(a = 1, 8, b = 1))
 
   bold(ht_orig) <- TRUE
   ht <- insert_column(ht_orig, 8, 9, after = 1)
